<?php
namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO\Response\Error;

class Fai04100CalculoDeApuracaoDeRetroativos extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName = "fai04100_ctrlapurfat_cliente";
    protected $parameterBagColumns = array("CDCLIENTE", "DTINIAPURAC", "DTFIMAPURAC", "CDFILIAL");

    public function __construct($crudService, $parameterBag, $connection){
        $this->connection = $connection;
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
    }

    public function procedureCalculo($request, $response) {

        $register = $request->getRow();
        $rows = $register['rows'];
        $dtproc = $this->getDtProc();
        $dtproc = substr($dtproc, 0, 10);
        $successProcedure = 0;
        foreach($rows as $row){
            $procedure = new StoredProcedure($this->connection, 'APURAC_RETROATIVO_GD');

            $procedure->addParam(new Param('CDFILIFATU',   Param::PARAM_INPUT, $row['CDFILIAL']));
            $procedure->addParam(new Param('NRAPURACFAT',  Param::PARAM_INPUT, $row['NRAPURACFAT']));
            $procedure->addParam(new Param('DTINIAPURAC',  Param::PARAM_INPUT, substr($row['DTINIAPURAC'], 0, 10)));
            $procedure->addParam(new Param('DTFIMAPURAC',  Param::PARAM_INPUT, substr($row['DTFIMAPURAC'], 0, 10)));
            $procedure->addParam(new Param('CDFILIAL',     Param::PARAM_INPUT, $register['CDFILIAL']));
            $procedure->addParam(new Param('DTPROC',       Param::PARAM_INPUT, $dtproc));
            $procedure->addParam(new Param('PRXAPURACFAT', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 400));

            $retorno = $procedure->execute();
            $successProcedure = empty($retorno['PRXAPURACFAT']) ? $successProcedure + 1 : $successProcedure;
        }

        if($successProcedure > 0){
            $response->setError(new Error("Não foi gerada apuração retroativa para algumas apurações.", 500));
        }
    }

    public function getDtProc(){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $this->crudService->getEnvironment()->getCdFilial()
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->crudService->getEnvironment()->getNrOrg()
            )
        );
        $dtprocessa = $this->crudService->getFilteredDataSet('paramfilial', $conditions)->getRows()[0];
        $dtprocessa['DTPROCESSA'] = $dtprocessa['DTPROCESSA'] ? $dtprocessa['DTPROCESSA'] : date('d/m/Y');
        return $dtprocessa['DTPROCESSA'];
    }
    
    public function cancelaApuracao (DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $notafiscal = $this->getNrLanctoNf($row);
        $this->deleteNF($notafiscal);
        $this->updateCtrlApurFat($row);
    }
    
    public function getNrLanctoNf($row){
        $sql = 'SELECT NRNOTAFISC, CDFILIAL, NRLANCTONF FROM NOTAFISCAL WHERE CDFILIALRETRO = :P_CDFILIALRETRO AND NRAPURFATNFRETR = :P_NRAPURFATNFRETR';
        $params = array(
            'P_CDFILIALRETRO' => $row['CDFILIALRETRO'],
            'P_NRAPURFATNFRETR' => $row['NRAPURFATRETRO']
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetch();
    }
    
    public function deleteNF($row) {
        $procedure = new StoredProcedure($this->connection, 'DELETA_NF');
        $procedure->addParam(new Param('P_FILIAL',   Param::PARAM_INPUT, $row['CDFILIAL']));
        $procedure->addParam(new Param('P_LANCTONF',  Param::PARAM_INPUT, $row['NRLANCTONF']));
        $procedure->addParam(new Param('P_IDCORPITEM',  Param::PARAM_INPUT, null));
        $procedure->execute();
    }
    
    public function updateCtrlApurFat($row){
        $sql = 'UPDATE CTRLAPURFAT SET CDFILIALRETRO = NULL, NRAPURFATRETRO = NULL WHERE CDFILIAL = :P_CDFILIALFATU AND NRAPURACFAT = :P_NRAPURACFAT';
        $params = array(
            'P_CDFILIALFATU' => $row['CDFILIAL'],
            'P_NRAPURACFAT' => $row['NRAPURACFAT']
        );
        $this->connection->beginTransaction();
        try {
            $stmt = $this->connection->prepare($sql);
            $stmt->execute($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}